//
//  KeyValueView.m
//  Locations
//
//  Created by Ray Kiddy on 7/2/09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "KeyValueView.h"

#import "LocationsAppDelegate.h"

#import "Event.h"

@implementation KeyValueView

@synthesize keyLabel;
@synthesize valueLabel;

@synthesize keyField;
@synthesize valueField;

@synthesize resetButton;
@synthesize saveButton;

@synthesize attributeName;

@synthesize event;
@synthesize originalValue;


- (void)saveAction:(id)sender {
	// First, we have to check that both fields are non-empty.
	if ([keyField.text length] > 0 && [valueField.text length] > 0) {
		// Second, we have to do a setObject, but only for an "extra" attribute.
		if ([[[event entity] attributesByName] objectForKey:keyField.text] == nil)
			[event setValue:valueField.text	forKey:keyField.text];
	}
	[[[LocationsAppDelegate delegate] managedObjectContext] save:nil];
}

- (void)resetAction:(id)sender {
	[[[LocationsAppDelegate delegate] managedObjectContext] rollback];
}

- (void)layoutSubviews {

	keyLabel = [[UILabel alloc] init];
	keyLabel.frame = CGRectMake(20.0f, 20.0f, 70.0f, 30.0f);
	keyLabel.text = [NSString stringWithFormat:@"key:"];

	keyField = [[UITextField alloc] init];
	keyField.frame = CGRectMake(90.0f, 20.0f, 200.0f, 30.0f);
	keyField.borderStyle = UITextBorderStyleRoundedRect;
	keyField.delegate = self;
	
	keyField.text = self.attributeName;

	valueLabel = [[UILabel alloc] init];
	valueLabel.frame = CGRectMake(20.0f, 60.0f, 70.0f, 30.0f);
	valueLabel.text = @"value = ";

	valueField = [[UITextField alloc] init];
	valueField.frame = CGRectMake(90.0f, 60.0f, 200.0f, 30.0f);
	valueField.borderStyle = UITextBorderStyleRoundedRect;
	valueField.delegate = self;

	valueField.text = [[self.event valueForKey:self.attributeName] description];

	saveButton = [[UIButton buttonWithType:UIButtonTypeRoundedRect] retain];
	saveButton.frame = CGRectMake(20.0f, 320.0f, 280.0f, 30.0f);

	[saveButton setTitle:@"Save" forState:UIControlStateNormal];
	[saveButton setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
	[saveButton setTitleColor:[UIColor grayColor] forState:UIControlStateDisabled];
	[saveButton addTarget:self action:@selector(saveAction:) forControlEvents:UIControlEventTouchUpInside];

	resetButton = [[UIButton buttonWithType:UIButtonTypeRoundedRect] retain];
	resetButton.frame = CGRectMake(20.0f, 360.0f, 280.0f, 30.0f);

	[resetButton setTitle:@"Reset" forState:UIControlStateNormal];
	[resetButton setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
	[resetButton setTitleColor:[UIColor grayColor] forState:UIControlStateDisabled];
	[resetButton addTarget:self action:@selector(resetAction:) forControlEvents:UIControlEventTouchUpInside];

	[self insertSubview:keyLabel atIndex:0]; [keyLabel release];

	[self insertSubview:keyField atIndex:1]; [keyField release];

	[self insertSubview:valueLabel atIndex:2]; [valueLabel release];

	[self insertSubview:valueField atIndex:3]; [valueField release];

	[self insertSubview:saveButton atIndex:4]; [saveButton release];

	[self insertSubview:resetButton atIndex:5]; [resetButton release];

	self.originalValue = [[event valueForKey:self.attributeName] description];
}

- (void)textFieldDidEndEditing:(UITextField *)textField {

	static NSDateFormatter * dateFormatter = nil;

    if (dateFormatter == nil) {
        dateFormatter = [[NSDateFormatter alloc] init];
        [dateFormatter setTimeStyle:NSDateFormatterMediumStyle];
        [dateFormatter setDateStyle:NSDateFormatterMediumStyle];
    }

	static NSNumberFormatter * decimalFormatter = nil;

    if (decimalFormatter == nil) {
        decimalFormatter = [[NSNumberFormatter alloc] init];
        [decimalFormatter setNumberStyle:NSNumberFormatterDecimalStyle];
    }

	switch ([[[[event entity] attributesByName] objectForKey:self.attributeName] attributeType]) {
		case NSDoubleAttributeType:
			[self.event setValue:[decimalFormatter numberFromString:self.valueField.text] forKey:self.attributeName];
			break;
		case NSStringAttributeType:
			[self.event setValue:self.valueField.text forKey:self.attributeName];
			break;
		case NSDateAttributeType:
			[self.event setValue:[dateFormatter dateFromString:self.valueField.text] forKey:self.attributeName];
			break;
		default:
			break;
	}
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField {
	[textField resignFirstResponder];
	return NO;
}

- (void)dealloc {	
    [super dealloc];
}

@end
