//
//  LocationsAppDelegate.m
//  Locations
//
//  Created by Ray Kiddy on 7/2/09.
//  Copyright __MyCompanyName__ 2009. All rights reserved.
//

#import "LocationsAppDelegate.h"

#import "RootViewController.h"
#import "KeyValueController.h"

@implementation LocationsAppDelegate

@synthesize window;
@synthesize navigationController;

+ (LocationsAppDelegate *)delegate {
	return (LocationsAppDelegate *)[[UIApplication sharedApplication] delegate];
}

- (void)send:(NSArray *)objects {
	
}

#pragma mark -
#pragma mark Application lifecycle

- (void)applicationDidFinishLaunching:(UIApplication *)application {
	
    // Configure and show the window.
	
    RootViewController *rootViewController = [[RootViewController alloc] initWithStyle:UITableViewStylePlain];
	
    NSManagedObjectContext *context = [self managedObjectContext];
    if (!context) {
        // Handle the error.
    }

    // Pass the managed object context to the view controller.
    rootViewController.managedObjectContext = context;
	
    UINavigationController *aNavigationController = [[UINavigationController alloc] initWithRootViewController:rootViewController];
    self.navigationController = aNavigationController;
	
    [window addSubview:[navigationController view]];
    [window makeKeyAndVisible];
	
    [rootViewController release];
    [aNavigationController release];
}

/**
 applicationWillTerminate: saves changes in the application's managed object context before the application terminates.
 */
- (void)applicationWillTerminate:(UIApplication *)application {
	
    NSError *error;
    if (managedObjectContext != nil) {
        if ([managedObjectContext hasChanges] && ![managedObjectContext save:&error]) {
			// Handle error
			NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
			exit(-1);  // Fail
        } 
    }
}


#pragma mark -
#pragma mark Saving

/**
 Performs the save action for the application, which is to send the save:
 message to the application's managed object context.
 */
- (IBAction)saveAction:(id)sender {
	
    NSError *error;
    if (![[self managedObjectContext] save:&error]) {
		// Handle error
		NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
		exit(-1);  // Fail
    }
}


#pragma mark -
#pragma mark Core Data stack

/**
 Returns the managed object context for the application.
 If the context doesn't already exist, it is created and bound to the persistent store coordinator for the application.
 */
- (NSManagedObjectContext *) managedObjectContext {
	
    if (managedObjectContext != nil) {
        return managedObjectContext;
    }
	
    NSPersistentStoreCoordinator *coordinator = [self persistentStoreCoordinator];
    if (coordinator != nil) {
        managedObjectContext = [[NSManagedObjectContext alloc] init];
        [managedObjectContext setPersistentStoreCoordinator: coordinator];
    }
    return managedObjectContext;
}


/**
 Returns the managed object model for the application.
 If the model doesn't already exist, it is created by merging all of the models found in the application bundle.
 */
- (NSManagedObjectModel *)managedObjectModel {
	
    if (managedObjectModel != nil) {
        return managedObjectModel;
    }
    managedObjectModel = [[NSManagedObjectModel mergedModelFromBundles:nil] retain];    
    return managedObjectModel;
}


/**
 Returns the persistent store coordinator for the application.
 If the coordinator doesn't already exist, it is created and the application's store added to it.
 */
- (NSPersistentStoreCoordinator *)persistentStoreCoordinator {
	
    if (persistentStoreCoordinator != nil) { return persistentStoreCoordinator; }
	
    NSURL *storeUrl = [NSURL fileURLWithPath: [[self applicationDocumentsDirectory] stringByAppendingPathComponent: @"Locations.sqlite"]];
	
	NSError *error;
	NSDictionary * options = [NSDictionary dictionaryWithObjectsAndKeys: [NSNumber numberWithBool:YES], NSMigratePersistentStoresAutomaticallyOption,
																		 [NSNumber numberWithBool:YES], NSInferMappingModelAutomaticallyOption,
																		 nil];
	persistentStoreCoordinator = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel: [self managedObjectModel]];
    if (![persistentStoreCoordinator addPersistentStoreWithType:NSSQLiteStoreType
												  configuration:nil
															URL:storeUrl
														options:options
														  error:&error]) {
        // Handle error
    }    
	
    return persistentStoreCoordinator;
}

//- (NSArray *)eventAttributeNames {

//	NSMutableArray * attributes = [NSMutableArray arrayWithArray:[[[[[self managedObjectModel] entitiesByName] objectForKey:@"Event"] attributesByName] allKeys]];
//
//	NSEntityDescription * extrasEntity = [[[self managedObjectModel] entitiesByName] objectForKey:@"EventExtra"];
//	
//	NSFetchRequest * request = [[NSFetchRequest alloc] init];
//	
//	[request setEntity:extrasEntity];
//	
//	NSError * error;
//	NSArray * fetchResults = [self managedObjectContext] executeFetchRequest:request error:&error];
//	
//	NSEnumerator * e = [fetchResults objectEnumerator]; NSManagedObject * obj;
	
//	while (obj = [e nextObject]) {
//		if ([obj valueForKey:@"name"] != nil)
//			[attributes addObject:[obj valueForKey:@"name"]];
//	}
//	
//	if (! [[NSSet setWithArray:self.attributeNames] isEqual: attributes]) {
//		[attributes minusSet: [NSSet setWithArray:self.attributeNames]];
//		NSMutableArray * newAttributes = [[NSMutableArray alloc] init];
//		[newAttributes addObjectsFromArray:self.attributeNames];
//		[newAttributes addObjectsFromArray:[attributes allObjects]];
//		[self setAttributeNames:newAttributes];
//	}
//}	

#pragma mark -
#pragma mark Application's documents directory

/**
 Returns the path to the application's documents directory.
 */
- (NSString *)applicationDocumentsDirectory {
	
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *basePath = ([paths count] > 0) ? [paths objectAtIndex:0] : nil;
    return basePath;
}


#pragma mark -
#pragma mark Memory management

- (void)dealloc {
	
    [managedObjectContext release];
    [managedObjectModel release];
    [persistentStoreCoordinator release];
    
	[window release];
	[super dealloc];
}


@end

