//
//  JSRandomCrazyPageView.m
//  BigViewThing
//
//  Created by Jonathan Saggau on 7/3/09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "BigViewPageView.h"
#import "NSOperationQueue+PLBlocks.h"
#import "BigViewThingAppDelegate.h"

CGFloat scale = .94;

@interface BigViewPageView ()

@property(retain)id offscreenContext;

@end

@interface BigViewPageView (privateAPI)
-(void)createOffscreenCtx;
-(void)initOffscreenContext;
@end

@implementation BigViewPageView

- (id)initWithFrame:(CGRect)frame {
    if (self = [super initWithFrame:frame]) {
        pageToDraw = 1;
        drawingSuspended = YES;
        lastFrameSize = frame.size;
        [self setBackgroundColor:[UIColor lightGrayColor]];
        [self setContentMode:UIViewContentModeScaleToFill];
        [self setClearsContextBeforeDrawing:NO];
        imageName = nil;
        [self initOffscreenContext];
    }
    return self;
}

-(void)initOffscreenContext
{
    CGSize layerSize = [self bounds].size;
    layerSize.height = floorf(layerSize.height);
    layerSize.width = floorf(layerSize.width);
    
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
    CGContextRef ctx = (CGContextRef) [(id) CGBitmapContextCreate(NULL, layerSize.width, layerSize.height, 8, layerSize.width*4, colorSpace, kCGImageAlphaPremultipliedLast) autorelease];
    CGColorSpaceRelease(colorSpace);
    CGContextTranslateCTM(ctx, 0, layerSize.height);
    CGContextScaleCTM(ctx, 1.0, -1.0);
    
    CGFloat tx = layerSize.width * (1.0 - scale) * 0.5;
    CGFloat ty = layerSize.height * (1.0 - scale) * 0.5;
    CGRect tempbounds = CGRectZero;
    tempbounds.size = layerSize;
    tempbounds = CGRectIntegral(CGRectInset(tempbounds, tx, ty));
    CGContextSetShadow(ctx, CGSizeMake(5,5), 5);
    CGContextSetFillColorWithColor(ctx, [[UIColor whiteColor] colorWithAlphaComponent:0.5].CGColor);
    CGContextFillRect(ctx, tempbounds);
    self.offscreenContext = (id) ctx;
}

-(void)setNeedsDisplay
{    
    if(!drawingSuspended &&
       (offscreenContext == NULL || shouldRedraw)&& 
       !IsEmpty(imageName))
    {
        [self createOffscreenCtx];
        shouldRedraw = NO;
    } 
    else
    {
        [super setNeedsDisplay];
    }
}

-(void)setFrame:(CGRect )someFrame
{
    if(!drawingSuspended)
    {
        if(!CGSizeEqualToSize(someFrame.size, lastFrameSize))// && !drawingSuspended)
        {
            NSLog(@"Resetting due to changed size");
            shouldRedraw = YES;
            lastFrameSize = someFrame.size;
            [self setNeedsDisplay];
        }
    }
    
    [super setFrame:someFrame];
}

-(void)createOffscreenCtx
{
    NSOperationQueue *q = [(BigViewThingAppDelegate *) [[UIApplication sharedApplication] delegate] globalQ];
    //[anOffscreenOperation setQueuePriority:drawingSuspended ? NSOperationQueuePriorityLow : NSOperationQueuePriorityNormal];
    PLBlockOperation *op = [PLBlockOperation blockOperationWithBlock:^{
        //imgRef = [[UIImage imageNamed:imageName] CGImage];
        NSString* bundlePath = [[NSBundle mainBundle] bundlePath];
        UIImage *img = [UIImage imageWithContentsOfFile:[NSString stringWithFormat:@"%@/%@", bundlePath, imageName]];
        CGImageRef imgRef = [img CGImage];
        
        CGSize layerSize = [self bounds].size;
        layerSize.height = floorf(layerSize.height);
        layerSize.width = floorf(layerSize.width);
        CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
        CGContextRef ctx = (CGContextRef) [(id) CGBitmapContextCreate(NULL, layerSize.width, layerSize.height, 8, layerSize.width*4, colorSpace, kCGImageAlphaPremultipliedLast) autorelease];
        CGColorSpaceRelease(colorSpace);
        CGContextTranslateCTM(ctx, 0, layerSize.height);
        CGContextScaleCTM(ctx, 1.0, -1.0);
        
        CGFloat tx = layerSize.width * (1.0 - scale) * 0.5;
        CGFloat ty = layerSize.height * (1.0 - scale) * 0.5;
        CGRect tempbounds = CGRectZero;
        tempbounds.size = layerSize;
        tempbounds = CGRectIntegral(CGRectInset(tempbounds, tx, ty));
        CGContextSetShadow(ctx, CGSizeMake(5,5), 5);
#if TARGET_IPHONE_SIMULATOR
        sleep(2.5);
#endif
        CGContextDrawImage(ctx, tempbounds, imgRef);
        self.offscreenContext = [[(id) ctx retain] autorelease];
        NSLog(@"Image loaded for %d", pageToDraw);
        [self performSelectorOnMainThread:@selector(setNeedsDisplay) withObject:nil waitUntilDone:NO];
    }];
    [op setInterestedObject:self];
    [q addOperation:op];
}

-(void)drawRect:(CGRect)rect
{   
    //NSLog(@"drawRect");
    CGContextRef context = UIGraphicsGetCurrentContext();    
    CGContextRef osc = (CGContextRef) self.offscreenContext;
    UIGraphicsPushContext(osc);
    CGImageRef tempImage = CGBitmapContextCreateImage (osc);
    UIGraphicsPopContext();
    if(tempImage)
    {
        //NSLog(@"drawing %@ in bounds %@ (page %d) offscreenContext %p context %p", imageName, NSStringFromCGRect(self.bounds), pageToDraw, self.offscreenContext, context);
        CGContextDrawImage(context, self.bounds, tempImage);
        CGImageRelease(tempImage);
        drawnPageOnce = YES;
    }
    else {
        NSLog(@"grrrr This shouldn't happen %d", pageToDraw);
    }
}

-(void)didMoveToSuperview
{
    //NSLog(@"didMoveToSuperview %d", pageToDraw);
    drawnPageOnce = NO;
    [self setNeedsDisplay];
}

-(NSString *)description
{
    NSString *des = [super description];
    des = [des stringByAppendingFormat:@" pageToDraw = %d imageName = %@", pageToDraw, imageName];
    return des;
}

#pragma mark accessors

@synthesize pageToDraw;
- (void)setPageToDraw:(NSUInteger)aPageToDraw
{
    //NSLog(@"in -setPageToDraw, old value of pageToDraw: %d, changed to: %d", pageToDraw, aPageToDraw);
    @synchronized(self)
    {
        if(pageToDraw != aPageToDraw)
        {
            pageToDraw = aPageToDraw;
            drawnPageOnce = NO;
        }
    }
}

@synthesize drawingSuspended;
- (void)setDrawingSuspended:(BOOL)flag
{
    @synchronized(self){
        if (drawingSuspended != flag) {
            //NSString *sus = flag ? @"SUSPEND" : @"RESUME";
            //NSLog(@"%@ %d", sus, pageToDraw);
            drawingSuspended = flag;
            if(!drawingSuspended)
            {
                [self setNeedsDisplay];
            }
            [self setFrame:self.frame];
        }
    }
}

@synthesize imageName;
- (void)setImageName:(NSString *)anImageName
{
    @synchronized(self) {
        if (![imageName isEqualToString:anImageName]) {
            NSLog(@"in  -setImageName:, old value of imageName: %@, changed to: %@", imageName, anImageName);
            
            anImageName = [anImageName copy];
            [imageName release];
            imageName = anImageName;
            if(!IsEmpty(imageName))
            {
                self.offscreenContext = nil;
                [self initOffscreenContext];
                [self setNeedsDisplay];
            }
        }
    }
}

@synthesize offscreenContext;

- (void)dealloc {
    self.offscreenContext = nil;
    self.imageName = nil;
    [super dealloc];
}

@end

