//
//  NSOperation+Blocks.m
//  NSOperationBlocks
//
//  Created by Landon Fuller on 7/4/09.
//  Copyright 2009 Plausible Labs Cooperative, Inc.. All rights reserved.
//

#import "NSOperationQueue+PLBlocks.h"

/**
 * Add additional block support to NSOperationQueue. Category
 * methods use PL-prefixed methods to avoid naming
 * conflicts with future OS releases.
 */
@implementation NSOperationQueue (PLBlocks)

/**
 * Add a block-based operation to the queue.
 */
- (void) pl_addOperationWithBlock: (void (^)()) block {
    [self addOperation: [PLBlockOperation blockOperationWithBlock: block]];
}

- (void)cancelOperationsFilteredByPredicate:(NSPredicate *)predicate;
{
    NSArray *ops = [[self operations] filteredArrayUsingPredicate:predicate];
    for (NSOperation *op in ops) {
        if(![op isExecuting] && ![op isFinished] && ![op isCancelled])
        {
            NSLog(@"cancelling %@", op);
            [op cancel];
        } else
        {
            NSLog(@"Left this one alone %@", op);
        }
    }
}

@end


/**
 * Wraps a block as an NSOperation
 */

@implementation PLBlockOperation

@synthesize interestedObject;

/** Create a new operation with the provided block. */
+ (id) blockOperationWithBlock: (void (^)()) block {
    return [[[self alloc] initWithBlock: block] autorelease];
}

/** Initialize with the provided block. */
- (id) initWithBlock: (void (^)()) block {
    if ((self = [super init]) == nil)
        return nil;
    
    /* Blocks must be copied */
    _block = [block copy];
    
    return self;
}

-(void)start
{
//    LogMethod();
    [super start];
}

-(void)cancel
{
    LogMethod();
    [super cancel];
}

// from NSOperation
- (void) main {
    LogMethod();
    //NSAssert((![self isCancelled] && ![self isFinished] && ![self isExecuting]), @"main should not be called if we're finished, cancelled, or executing");
    _block();
}

-(NSString *)description
{
    NSString *des = [super description];
    NSString *runnung = [self isExecuting] ? @"RUNNING" : @"NOT running";
    NSString *cancelled = [self isCancelled] ? @"CANCELLED" : @"NOT cancelled";
    NSString *finished = [self isFinished] ? @"FINISHED" : @"NOT finished";
    des = [des stringByAppendingFormat:@"   \n   interestedObj = %@ %@ %@ %@", interestedObject, runnung, cancelled, finished];
    return des;
}

- (void) dealloc {
    interestedObject = nil;
    [_block release];
    [super dealloc];
}

@end