//
//  BBInputViewController.m
//  BBOpenGLGameTemplate
//
//  Created by ben smith on 1/07/09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "BBInputViewController.h"
#import "BBInterfaceObject.h"
#import "BBRenderController.h"

@implementation BBInputViewController

@synthesize touchEvents, forwardMagnitude, rightMagnitude, leftMagnitude,fireMissile;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
	if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
		// init our touch storage set
		touchEvents = [[NSMutableSet alloc] init];
		touchesToDistribute = [[NSMutableArray alloc] init];
		forwardMagnitude = 0.0;
		leftMagnitude = 0.0;
		rightMagnitude = 0.0;
	}
	return self;
}

-(void)loadView
{
	
}

-(CGRect)screenRectFromMeshRect:(CGRect)rect atPoint:(CGPoint)meshCenter
{
	// find the point on the screen that is the center of the rectangle
	// and use that to build a screen-space rectangle
	CGPoint screenCenter = CGPointZero;
	CGPoint rectOrigin = CGPointZero;
	// since our view is rotated, then our x and y are flipped
	screenCenter.x = meshCenter.y + 160.0; // need to shift it over
	screenCenter.y = meshCenter.x + 240.0; // need to shift it up
	
	rectOrigin.x = screenCenter.x - (CGRectGetHeight(rect)/2.0); // height and width 
	rectOrigin.y = screenCenter.y - (CGRectGetWidth(rect)/2.0); // are flipped
	
	return CGRectMake(rectOrigin.x, rectOrigin.y, CGRectGetHeight(rect), CGRectGetWidth(rect));
}

+(BBPoint)gamePointFromScreenPoint:(CGPoint)p
{
	BBPoint gamePoint;
	gamePoint.x = p.y - 240.0;
	gamePoint.y = p.x - 160.0;
	gamePoint.z = 0;
	return gamePoint;
}


#pragma mark Touch Event Handlers

// just a handy way for other object to clear our events
- (void)clearEvents
{
	[touchEvents removeAllObjects];
}


- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
	// just store them all in the big set.
	[touchEvents addObjectsFromArray:[touches allObjects]];
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
	// just store them all in the big set.
	[touchEvents addObjectsFromArray:[touches allObjects]];
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
	// just store them all in the big set.
	[touchEvents addObjectsFromArray:[touches allObjects]];
}

- (void)distributeTouches
{
	BOOL didFindOne = NO;
	[touchesToDistribute removeAllObjects];
	[touchesToDistribute addObjectsFromArray:[touchEvents allObjects]];
	for (BBInterfaceObject * button in interfaceObjects) {
		didFindOne = NO;
		for (UITouch * touch in touchesToDistribute) {
			CGPoint touchPoint = [touch locationInView:[touch view]];
			if (CGRectContainsPoint(button.screenRect, touchPoint)) {
				didFindOne = YES;
				[button addTouchEvent:touch];
			}
		}
		//if (didFindOne) [touchesToDistribute removeObjectsInArray:[button.touchEvents allObjects]];
	}
}


#pragma mark view controller stuff

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}

#pragma mark Loading the Buttons

-(void)clearInterface
{
	[interfaceObjects removeAllObjects];
}

-(void)addInterfaceObject:(BBInterfaceObject*)interfaceObject
{
	if (interfaceObjects == nil) interfaceObjects = [[NSMutableArray alloc] init];
	[interfaceObjects addObject:interfaceObject];	
}

-(void)removeInterfaceObject:(BBInterfaceObject*)interfaceObject
{
	[interfaceObjects removeObject:interfaceObject];
}

-(void)updateInterface:(NSTimeInterval)deltaTime
{
	// need to send the touches to the buttons that got hit
	[self distributeTouches];
	for (BBSceneObject * uiObject in interfaceObjects) [uiObject update:deltaTime];
}

-(void)renderInterface:(BBRenderController*)renderer
{
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	glRotatef(-90.0f, 0.0f, 0.0f, 1.0f);
	
	// set up the viewport so that it is analagous to the screen pixels
	glOrthof(-240, 240, -160, 160, -1.0f, 50.0f);
	
	glMatrixMode(GL_MODELVIEW);
	glDisable(GL_LIGHTING);
	glDisable(GL_CULL_FACE);
	// simply call 'render' on all our scene objects
	[renderer renderObjects:interfaceObjects];

	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
}

- (void)dealloc 
{
	[touchEvents release];
	[touchesToDistribute release];
	[super dealloc];
}


@end

